﻿using MPKICrypto;
using Org.BouncyCastle.Security;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.Configuration;
using System.Web.Script.Serialization;
using System.Web.Services;

namespace TestWebService
{
    /// <summary>
    /// Summary description for SignPDF
    /// </summary>
    [WebService(Namespace = "http://tempuri.org/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [System.ComponentModel.ToolboxItem(false)]
    // To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
    // [System.Web.Script.Services.ScriptService]
    public class SignPDF : System.Web.Services.WebService
    {
        public class RV
        {
            public String ValidationCode { get; set; }
            public String FileID { get; set; }
        }

        [WebMethod]
        public byte[] Sign(byte[] inPdf, String messageToBeDisplayed, String phone, String signerName, byte[] imgBuff, int width, int height)
        {
            Authorization.SetKey(WebConfigurationManager.AppSettings["LICENSE_KEY"]);

            String apid = "http://ap.mobile-id.vn/viettelca";
            String appwd = "12345678";
            String temp = Path.GetTempFileName();
            Image sigImg = null;

            String filename = base.Server.MapPath(String.Format("~/files/{0}.pdf", Guid.NewGuid().ToString("D")));
            String outPdf = Path.Combine(Path.GetDirectoryName(filename), String.Format("{0}.signed.pdf", Path.GetFileNameWithoutExtension(filename)));

            try
            {
                File.WriteAllBytes(temp, imgBuff);
                sigImg = Bitmap.FromFile(temp);

                //String filename = base.Server.MapPath(String.Format("~/files/{0}.pdf", Guid.NewGuid().ToString("D")));
                //String outPdf = Path.Combine(Path.GetDirectoryName(filename), String.Format("{0}.signed.pdf", Path.GetFileNameWithoutExtension(filename)));
                File.WriteAllBytes(filename, inPdf);


                PdfSigner pdf = new PdfSigner(filename, outPdf, phone, signerName);
                pdf.AllowOnlineCheckingCert = true;
                pdf.AllowCheckingCertViaOCSP = true;
                //Trường hợp dùng AP
                pdf.MSSPMode = false;

                ////Trường hợp dùng MSSP
                //pdf.MSSPMode = true;
                //pdf.APCertificate = apCert;
                //pdf.APCertPassword = apCertPwd;
                //pdf.APID = "http://ap.mobile-id.vn/viettelca";
                //pdf.APPWD = "12345678";

                pdf.MessageToBeDisplayed = messageToBeDisplayed;
                pdf.SignatureAppearance = iTextSharp.text.pdf.PdfSignatureAppearance.RenderingMode.GRAPHIC;
                pdf.SignatureImage = sigImg;
                pdf.SignatureWidth = width;
                pdf.SignatureHeight = height;
                pdf.TsaUrl = "http://ca.gov.vn/tsa";


                pdf.SignApprove();
                Console.WriteLine("OK");

                return File.ReadAllBytes(outPdf);
            }
            catch (ArgumentException ex)
            {
                Console.WriteLine("Loi tham so ky: {0}", ex.Message);
            }
            catch (GetCertException ex)
            {
                Console.WriteLine("Loi ket noi AP/MSSP: {0}", ex.Message);
            }
            catch (SignatureException ex)
            {
                Console.WriteLine("Loi ky so: {0}", ex.Message);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Loi tien trinh: {0}", ex.Message);
            }
            finally
            {
                if (sigImg != null) sigImg.Dispose();
                if (File.Exists(temp)) File.Delete(temp);
                if (File.Exists(filename)) File.Delete(filename);
                if (File.Exists(outPdf)) File.Delete(outPdf);
            }


            return null;
        }

        [WebMethod]
        public String CalculateVC(byte[] inPdf, String phone, String signerName, byte[] imgBuff, int width, int height)
        {
            Authorization.SetKey(WebConfigurationManager.AppSettings["LICENSE_KEY"]);

            String apid = "http://ap.mobile-id.vn/viettelca";
            String appwd = "12345678";
            String temp = Path.GetTempFileName();
            Image sigImg = null;

            String fileId = Guid.NewGuid().ToString("D");
            String filename = base.Server.MapPath(String.Format("~/files/{0}.pdf", fileId));
            String outPdf = Path.Combine(Path.GetDirectoryName(filename), String.Format("{0}.temp.pdf", fileId));
            String fileIdPath = base.Server.MapPath(String.Format("~/files/{0}.json", fileId));

            try
            {
                File.WriteAllBytes(temp, imgBuff);
                sigImg = Bitmap.FromFile(temp);

                File.WriteAllBytes(filename, inPdf);

                PdfSigner pdf = new PdfSigner(filename, outPdf, phone, signerName);
                pdf.AllowOnlineCheckingCert = true;
                pdf.AllowCheckingCertViaOCSP = true;
                //Trường hợp dùng AP
                pdf.MSSPMode = false;

                ////Trường hợp dùng MSSP
                //pdf.MSSPMode = true;
                //pdf.APCertificate = apCert;
                //pdf.APCertPassword = apCertPwd;
                //pdf.APID = "http://ap.mobile-id.vn/viettelca";
                //pdf.APPWD = "12345678";

                pdf.MessageToBeDisplayed = "Calculate VC";
                pdf.SignatureAppearance = iTextSharp.text.pdf.PdfSignatureAppearance.RenderingMode.GRAPHIC;
                pdf.SignatureImage = sigImg;
                pdf.SignatureWidth = width;
                pdf.SignatureHeight = height;
                pdf.TsaUrl = "http://ca.gov.vn/tsa";


                ValidationInfo vi = pdf.CalculateVC();
                Console.WriteLine("OK");
                var serializer = new JavaScriptSerializer();
                String json = serializer.Serialize(vi);

                File.WriteAllText(fileIdPath, json);

                RV rv = new RV()
                {
                    ValidationCode = vi.ValicationCode,
                    FileID = fileId
                };

                return serializer.Serialize(rv);
            }
            catch (ArgumentException ex)
            {
                Console.WriteLine("Loi tham so ky: {0}", ex.Message);
            }
            catch (GetCertException ex)
            {
                Console.WriteLine("Loi ket noi AP/MSSP: {0}", ex.Message);
            }
            catch (SignatureException ex)
            {
                Console.WriteLine("Loi ky so: {0}", ex.Message);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Loi tien trinh: {0}", ex.Message);
            }
            finally
            {
                if (sigImg != null) sigImg.Dispose();
                if (File.Exists(temp)) File.Delete(temp);
                if (File.Exists(filename)) File.Delete(filename);
            }


            return null;
        }

        [WebMethod]
        public byte[] SignFinal(String messageToBeDisplayed, String phone, String fileId)
        {
            Authorization.SetKey(WebConfigurationManager.AppSettings["LICENSE_KEY"]);

            String apid = "http://ap.mobile-id.vn/viettelca";
            String appwd = "12345678";
            String fileName = "";
            String fileTemp = "";
            String fileIdPath = base.Server.MapPath(String.Format("~/files/{0}.json", fileId));
            String outPdf = Path.Combine(Path.GetDirectoryName(fileIdPath), String.Format("{0}.signed.pdf", fileId));

            try
            {
                var serializer = new JavaScriptSerializer();
                ValidationInfo vi = serializer.Deserialize<ValidationInfo>(File.ReadAllText(fileIdPath));
                fileName = vi.FileName;
                fileTemp = vi.TempFileName;
                PdfSigner pdf = new PdfSigner(fileTemp, outPdf, phone);
                //Trường hợp dùng AP
                pdf.MSSPMode = false;

                ////Trường hợp dùng MSSP
                //pdf.MSSPMode = true;
                //pdf.APCertificate = apCert;
                //pdf.APCertPassword = apCertPwd;
                //pdf.APID = "http://ap.mobile-id.vn/viettelca";
                //pdf.APPWD = "12345678";

                pdf.MessageToBeDisplayed = messageToBeDisplayed;
                pdf.TsaUrl = "http://ca.gov.vn/tsa";


                pdf.SignFinal(vi);
                Console.WriteLine("OK");

                return File.ReadAllBytes(outPdf);
            }
            catch (ArgumentException ex)
            {
                Console.WriteLine("Loi tham so ky: {0}", ex.Message);
            }
            catch (GetCertException ex)
            {
                Console.WriteLine("Loi ket noi AP/MSSP: {0}", ex.Message);
            }
            catch (SignatureException ex)
            {
                Console.WriteLine("Loi ky so: {0}", ex.Message);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Loi tien trinh: {0}", ex.Message);
            }
            finally
            {
                if (File.Exists(outPdf)) File.Delete(outPdf);
                if (File.Exists(fileName)) File.Delete(fileName);
                if (File.Exists(fileIdPath)) File.Delete(fileIdPath);
                if (File.Exists(fileTemp)) File.Delete(fileTemp);
            }


            return null;
        }
    }
}
